/*******************************************************************************
 * Licensed Materials - Property of IBM and/or HCL
 * (c) Copyright IBM Corporation 2016. All Rights Reserved.
 * (c) Copyright HCL Technologies Ltd. 2017, 2021. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
console.log("rftjsproxies.js - document url -> " + window.top.document.URL);

var isFTEdge = false;
if ( navigator.userAgent.indexOf("Edge") != -1 ) {
	isFTEdge = true;
}

/*JSHtmlObject Proxy*/
var JSHtmlObject = function(obj) {
	this.obj = obj;
	this.nativeHandle = 0;
};

JSHtmlObject.prototype.getNativeHandle  = function() {
	if((this.nativeHandle==0) && this.obj.FtNativeHandle) {
		this.nativeHandle = this.obj.FtNativeHandle;
	}
	return this.nativeHandle;
};

JSHtmlObject.prototype.setNativeHandle  = function(handle) {
	this.obj.FtNativeHandle = handle;
	this.nativeHandle = handle;
};

JSHtmlObject.prototype.getStandardPropertynames = function() {
	try {
		var props = [];
		for(key in PropertySet) {
			if(!(key.startsWith("INTERNAL"))) {
				var propName = PropertySet[key];
				propName = "." + propName;
				var val = this.getProperty(propName);
				if(val!=null && val!=undefined) {
					props.push(propName);
				}
			}
		}
		return props;
	} catch(err){}
	return null;
}

JSHtmlObject.prototype.isStandardProp = function(propName) {
	try {
		if(propName.startsWith(".")) {
			propName = propName.substring(1, propName.length);
		}
		for(key in PropertySet) {
			if(PropertySet[key] == propName) {
				return true;
			}
		}
		return false;
	} catch(err){}
	return false;
};

JSHtmlObject.prototype.isPointFoundInObject = function(x, y) {
	try {
		var containsElem = false;
		var doc = this.getDocument();
		if(doc) {
			var elemAtPoint = doc.getChildAtPoint(x, y);
			while(elemAtPoint) {
				try {
					if(this.obj.isSameNode(elemAtPoint.obj)) {
						containsElem = true;
						break;
					}
				} catch(err){}
				elemAtPoint = elemAtPoint.getParent();
			}
		}
		return containsElem;
	} catch(err){
	}
	return false;
}; 

JSHtmlObject.prototype.getIntVal  = function(propName) {
	try {
		var propVal = parseInt(this.obj[propName]);
		if(!isNaN(propVal)) {
			return propVal;
		}
	} catch(err){}
	return 0;
};

JSHtmlObject.prototype.getBoolVal  = function(propName) {
	try {
		var propVal = this.obj[propName];
		if(typeof propVal == "boolean") {
			return propVal;
		}
	} catch(err){}
	return false;
};

JSHtmlObject.prototype.getStrVal  = function(propVal) {
	try {
		if(typeof propVal == "string") {
			return propVal;
		}
	} catch(err){}
	return null;
};

/*JSHtmlBrowser Proxy*/
var JSHtmlBrowser = function(obj) {
	JSHtmlObject.apply(this,obj);
	this.obj = obj;
};
JSHtmlBrowser.prototype = new JSHtmlObject();

JSHtmlBrowser.prototype.getDocumentHandle = function() {
	return null;
};

JSHtmlBrowser.prototype.getTopHtmlParent = function() {
	return this.nativeHandle;
};

JSHtmlBrowser.prototype.getDocument = function() {
	try {
		var doc = this.obj.document;
		if(doc) {
			return new JSHtmlDocument(doc);
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.getParent = function() {
	return null;
};

JSHtmlBrowser.prototype.getChildren = function() {
	try {
		var docObj = this.getDocument();
		if(docObj) {
			return [docObj];
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.getElementsByTag = function(tag) {
	return null;
};

JSHtmlBrowser.prototype.getElementWithFocus = function() {
	try {
		var docObj = this.getDocument();
		if(docObj) {
			return docObj.getElementWithFocus();
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.getChildAtPoint = function(x, y) {
	try {
		var docObj = this.getDocument();
		if(docObj) {
            var child = docObj.getChildAtPoint(x, y);
			if(child) {
				return child;
			}
			return docObj;
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.go = function(direction) {
	try {
		this.obj.history.go(direction);
		return true;
	} catch(err){}
	return false;
};

JSHtmlBrowser.prototype.loadUrl = function(url, target) {
	try {
		var expression = "^(https?|ftp|file)://.+$";
		if(url.match(expression) == null) {
			url = "http://" + url;
		}
		this.obj.document.location.assign(url);
		return true;
	} catch(err){}
	return false;
};

JSHtmlBrowser.prototype.scrollIntoView = function() {
	return true;
};

JSHtmlBrowser.prototype.scrollIntoViewWithDirection = function(direction) {
	return true;
};

JSHtmlBrowser.prototype.setProperty = function(propName, value) {
	return false;
};

JSHtmlBrowser.prototype.getElementsByXpath = function(xpath) {
	try {
		var docObj = this.getDocument();
		if(docObj) {
			return docObj.getElementsByXpath(xpath);
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.getElementsByJQuery = function(jquery) {
	try {
		var docObj = this.getDocument();
		if(docObj) {
			return docObj.getElementsByJQuery(jquery);
		}
	} catch(err){}
	return null;
};

JSHtmlBrowser.prototype.getProperties = function() {
	return null;
};

JSHtmlBrowser.prototype.getStandardProperties = function() {
	return this.getStandardPropertynames();
};

JSHtmlBrowser.prototype.isPointInObject = function(x, y) {
	return this.isPointFoundInObject(x, y);
};

JSHtmlBrowser.prototype.getZoomingFactor = function(win) {
	var zoomFactor = (win.outerWidth-9)/win.innerWidth;
	var zoomFactor1 = win.devicePixelRatio;
	if(zoomFactor1) {
		zoomFactor = zoomFactor1;
	}
	return zoomFactor;
};

JSHtmlBrowser.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName){
			case PropertySet.PROPERTYclass:
				propVal = "Html.HtmlBrowser";
				break;
			case PropertySet.PROPERTYbrowserName:
				propVal = "chrome";
				break;
			case PropertySet.INTERNALPROPERTYbrowserWinHandle:
				propVal = this.getNativeHandle();
				break;
			case PropertySet.PROPERTYreadyState:
				propVal = 0;
				var docObj = this.getDocument();
				if(docObj) {
					propVal = docObj.getReadyState();
				}
				break;
			case PropertySet.PROPERTYdocumentName:
				var doc = this.getDocument();
				if(doc) {
					propVal = doc.obj.URL;
				}
				break;
			case PropertySet.PROPERTYwindow:
				propVal = this.getNativeHandle();
				break;
			case PropertySet.PROPERTYbounds:
				propVal = [0,0,0,0];
				break;
			case PropertySet.PROPERTYclientRect:
				propVal = [0,0,0,0];
				break;
			case PropertySet.PROPERTYoffsetLeft:
				propVal = 0;
				break;
			case PropertySet.PROPERTYoffsetTop:
				propVal = 0;
				break;
			case PropertySet.PROPERTYoffsetWidth:
				propVal = this.getIntVal("innerWidth");
				break;
			case PropertySet.PROPERTYoffsetHeight:
				propVal = this.getIntVal("innerHeight");
				break;	
			case PropertySet.PROPERTYstatusText:
				propVal = this.obj.status;
				break;
			case PropertySet.PROPERTYbrowserWinRect:
				propVal = "x:"+this.obj.screenX+",y:"+this.obj.screenY+",w:"+this.obj.outerWidth+",h:"+this.obj.outerHeight;
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return null;
};

/*JSHtmlDocument Proxy*/
var JSHtmlDocument = function (obj) {
	JSHtmlObject.apply(this,obj);
	this.obj = obj;
	this.cachedReadyState = 0;
};
JSHtmlDocument.prototype = new JSHtmlObject();

JSHtmlDocument.prototype.getDocumentHandle = function() {
	return null;
};

JSHtmlDocument.prototype.getTopHtmlParent = function() {
	try {
		var topWin = window.top;
		if(topWin) {
			return new JSHtmlBrowser(topWin).getNativeHandle();
		}
	} catch(err){}
	return 0;
};

JSHtmlDocument.prototype.getAllFramesInsideRecursively = function() {
    try {
        var childFrames = [];
        var subChildFrames = [];
        var allFramesInCurrent = this.getAllFramesInCurrentDocument();
        for(var i=0; i<allFramesInCurrent.length; i++) {
            childFrames.push(allFramesInCurrent[i]);
            subChildFrames = allFramesInCurrent[i].getFrameDocument().getAllFramesInsideRecursively();
            for(var j=0; j<subChildFrames.length; j++) {
                childFrames.push(subChildFrames[j]);
            }
        }
        return childFrames;
    } catch(err){}
    return [];
};
JSHtmlDocument.prototype.getAllFramesInCurrentDocument = function() {
    try {
        var frameElements = [];
        var frames = this.getElementsByTag("frame");
        if(frames) {
            for(var i=0; i<frames.length; i++) {
                try {
                    var frameDoc = frames[i].getFrameDocument();
                    if(frameDoc) {
                        frameElements.push(frames[i]);
                    }
                } catch(err){}
            }
        }
        frames = this.getElementsByTag("iframe");
        if(frames) {
            for(var i=0; i<frames.length; i++) {
                try {
                    var frameDoc = frames[i].getFrameDocument();
                    if(frameDoc) {
                        frameElements.push(frames[i]);
                    }
                } catch(err){}
            }
        }
        return frameElements;
    } catch(err){}
    return [];
};
JSHtmlDocument.prototype.getParent = function() {
	var topWin = window.top;
	try {
		var doc = topWin.document;
		if(doc) {
			var docObj = new JSHtmlDocument(doc);
			var frames =docObj.getAllFramesInsideRecursively();
			if(frames) {
				for(var i=0; i<frames.length; i++) {
					var frameDoc = frames[i].getFrameDocument();
					try {
						if(this.obj.isSameNode(frameDoc.obj)) {
							return frames[i];
						}
					} catch(err){}
				}
			}
		}
	} catch(err){}
	return new JSHtmlBrowser(topWin);
};

JSHtmlDocument.prototype.getChildAtPoint = function(x, y) {
	var elem = null;
	if(this.obj) 
	{
		if(window.mozInnerScreenX != undefined && (this.getParent() instanceof JSHtmlBrowser) )
        {
			if(log) console.log("Adjust x and y for FF");
			x = x - mozInnerScreenX;
			y = y - mozInnerScreenY;
		} 
		else if (isFTEdge) 
		{
			if(log) console.log("Adjust x and y for Edge");
			x = x - (window.top.screenLeft || window.top.screenX || 0);
			y = y - (window.top.screenTop || window.top.screenY || 0);
		}  
		
		var zoomFactor = JSHtmlBrowser.prototype.getZoomingFactor(window.top);
		var offsetPt = this.convertFromScreenToOffset(this, {x:x, y:y}, zoomFactor);
        if(log)console.log("JSHtmlDocument, calling element from point with points "+ offsetPt.x+","+offsetPt.y);
		elem = this.obj.elementFromPoint(offsetPt.x, offsetPt.y);
		if(elem) 
		{
			return this.create(elem);
		}
	}
	return null;
};

JSHtmlDocument.prototype.getElementById = function(id) {
	try {
		var elem = this.obj.getElementById(id);
		if(elem) {
			return this.create(elem);
		}
	} catch(err){}
	return null;
};

JSHtmlDocument.prototype.getElementsByName = function(name) {
	try {
		var elems = this.obj.getElementsByName(name);
		if(elems) {
			var elem, result = [];
			for(var i=0; i<elems.length; i++) {
				if(elem=elems[i]) {
					result.push(this.create(elem));
				}
			}
			return result;
		}
	} catch(err){}
	return null;
};

JSHtmlDocument.prototype.getElementWithFocus = function() {
	try {
		var elem = this.obj.activeElement;
		if(elem) {
			return this.create(elem);
		}
	} catch(err){}
	return null;
};

JSHtmlDocument.prototype.getElementsByTag = function(tag) {
	try {
		var elems = this.obj.getElementsByTagName(tag);
		if(elems) {
			var elem, result = [];
			for(var i=0; i<elems.length; i++) {
				if(elem=elems[i]) {
					result.push(this.create(elem));
				}
			}
			return result;
		}
	} catch(err){}
	return null;
};

JSHtmlDocument.prototype.getAllDocumentsInsideRecursively = function() {
	try {
		var childDocs = [];
		var subChildDocs = [];
		var allDocsInCurrent = this.getAllDocumentHandlesInCurrentDocument();
		for(var i=0; i<allDocsInCurrent.length; i++) {
			childDocs.push(allDocsInCurrent[i]);
			subChildDocs = allDocsInCurrent[i].getAllDocumentsInsideRecursively();
			for(var j=0; j<subChildDocs.length; j++) {
				childDocs.push(subChildDocs[j]);
			}
		}
		return childDocs;
	} catch(err){}
	return [];
};

JSHtmlDocument.prototype.getAllDocumentHandlesInCurrentDocument = function() {
	try {
		var docs = [];
		var frames = this.getElementsByTag("frame");
		if(frames) {
			for(var i=0; i<frames.length; i++) {
				try {
					var frameDoc = frames[i].getFrameDocument();
					if(frameDoc) {
						docs.push(frameDoc);
					}
				} catch(err){}
			}
		}
		frames = this.getElementsByTag("iframe");
		if(frames) {
			for(var i=0; i<frames.length; i++) {
				try {
					var frameDoc = frames[i].getFrameDocument();
					if(frameDoc) {
						docs.push(frameDoc);
					}
				} catch(err){}
			}
		}
		return docs;
	} catch(err){}
	return [];
};

JSHtmlDocument.prototype.getChildren = function() {
	try {
		var docElem = this.obj.documentElement;
		if(docElem) {
			return [this.create(docElem)];
		} else {
			var docBody = this.obj.body;
			if(docBody) {
				return [this.create(docBody)];
			}
		}
	} catch(err){}
	return null;
};

JSHtmlDocument.prototype.scrollIntoView = function() {
	return true;
};

JSHtmlDocument.prototype.scrollIntoViewWithDirection = function(direction) {
	return true;
};

JSHtmlDocument.prototype.invokeScript = function(scriptMethod, scriptLang) {
	try {
		var ret = this.obj.defaultView.eval(scriptMethod);
		if(ret != undefined) {
			return ret;
		}
	} catch(err) {}
	return null;
};

JSHtmlDocument.prototype.setProperty = function(propName, value) {
	return false;
};

JSHtmlDocument.prototype.getElementsByXpath = function(xpath) {
	return JSHtmlElement.prototype.getAllElementsByXpath(xpath, this.obj, this.obj);
};

JSHtmlDocument.prototype.getElementsByJQuery = function(jquery) {
	return JSHtmlElement.prototype.getAllElementsByJQuery(jquery, this.obj, this.obj);
};

JSHtmlDocument.prototype.getProperties = function() {
	return null;
};

JSHtmlDocument.prototype.getStandardProperties = function() {
	return this.getStandardPropertynames();
};

JSHtmlDocument.prototype.isPointInObject = function(x, y) {
	return false;
};

JSHtmlDocument.prototype.create = function(elem) {
	if(elem.nodeType==3) {
		return new JSHtmlTextNodeElement(elem);
	}
	var tagName = elem.tagName;
	if(tagName)
	{
		if(tagName == "INPUT") {
			return new JSHtmlInputElement(elem);
		}
		if(tagName == "A") {
			return new JSHtmlAnchorElement(elem);
		}
		if(tagName == "SELECT") {
			return new JSHtmlSelectElement(elem);
		}
		if(tagName == "OPTION") {
			return new JSHtmlOptionElement(elem);
		}
		if(tagName == "IMG") {
			return new JSHtmlImageElement(elem);
		}
		if(tagName == "TEXTAREA") {
			return new JSHtmlTextAreaElement(elem);
		}
		if(tagName == "TABLE") {
			return new JSHtmlTableElement(elem);
		}
		if(tagName == "TR") {
			return new JSHtmlTableRowElement(elem);
		}
		if(tagName == "TD") {
			return new JSHtmlTableCellElement(elem);
		}
		if(tagName == "AREA") {
			return new JSHtmlAreaElement(elem);
		}
		if((tagName == "FRAME") || (tagName == "IFRAME")) {
			return new JSHtmlFrameElement(elem);
		}
		if(tagName == "FRAMESET") {
			return new JSHtmlFrameSetElement(elem);
		}
	}
	return new JSHtmlElement(elem);
};

JSHtmlDocument.prototype.getWindowOffsetPoint = function(doc) {
	var offsetPt = {x:0, y:0};
	try {
		var parent = doc.getParent();
		while(parent && !(parent instanceof JSHtmlBrowser)) {
			var frmOffstPt = parent.getFramesOffsetPoint();
			if(frmOffstPt) {
				offsetPt.x += frmOffstPt.x;
				offsetPt.y += frmOffstPt.y;
			}
			doc = parent.getDocumentHandle();
			if(doc) {
				parent = doc.getParent();
			} else {
				parent = null;
			}
		}
	} catch(err) {}
	return offsetPt;
};

JSHtmlDocument.prototype.convertFromOffsetToScreen = function(doc, pt, zoomFactor) {
	var offsetPt = {x:0, y:0};
	var winOffsetPt = this.getWindowOffsetPoint(doc);
	offsetPt.x = parseInt((pt.x + winOffsetPt.x) * zoomFactor);
	offsetPt.y = parseInt((pt.y + winOffsetPt.y) * zoomFactor);
	return offsetPt;
};

JSHtmlDocument.prototype.convertFromScreenToOffset = function(doc, pt, zoomFactor) {
	var offsetPt = {x:0, y:0};
	var winOffsetPt = this.getWindowOffsetPoint(doc);
	var parent = doc.getParent();
 	if( (winOffsetPt.x != 0 && winOffsetPt.y !=0 ) || (parent instanceof JSHtmlFrameElement) )
    {
 		if (window.mozInnerScreenX != undefined) {
			if(log) console.log("JSHtmlDocument : ConvertFromScreenToOffset , adding mozInnerScreenX and mozInnerScreenY to it "+ window.mozInnerScreenX+","+window.mozInnerScreenY);
			winOffsetPt.x += window.mozInnerScreenX;
			winOffsetPt.y += window.mozInnerScreenY;
 		} else if (isFTEdge) {
			if(log) console.log("JSHtmlDocument : Edge : ConvertFromScreenToOffset");
			winOffsetPt.x += (window.top.screenLeft || window.top.screenX || 0);
			winOffsetPt.y += (window.top.screenTop || window.top.screenY || 0);
 		}
    }
	offsetPt.x = parseInt((pt.x - winOffsetPt.x) / zoomFactor);
	offsetPt.y = parseInt((pt.y - winOffsetPt.y) / zoomFactor);
	return offsetPt;
};

JSHtmlDocument.prototype.getReadyState = function() {
	var docState = 0;
	try {
		if(this.cachedReadyState == 4) {
			docState = 4;
		} else {
			var readystate = this.obj.readyState;
			if(readystate == "complete") {
				var tempReadyState = "complete";
				var childDocs = this.getAllDocumentHandlesInCurrentDocument();
				for(var i=0; i<childDocs.length; i++) {
					tempReadyState = childDocs[i].obj.readyState;
					if(tempReadyState != "complete") {
						readystate = tempReadyState;
						break;
					}
				}
			}
			if(readystate == "uninitialized") {
				docState = 0;
			} else if(readystate == "loading") {
				docState = 1;
			} else if(readystate == "loaded") {
				docState = 2;
			} else if(readystate == "interactive") {
				docState = 3;
			} else if(readystate == "complete") {
				docState = 4;
			}
			if(docState == 4) {
				this.cachedReadyState = 4;
			}
		}
	} catch(err){}
	return docState;
};

JSHtmlDocument.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName){
			case PropertySet.PROPERTYclass:
				propVal = "Html.HtmlDocument";
				break;
			case PropertySet.PROPERTYreadyState:
				propVal = this.getReadyState();
				break;
			case PropertySet.PROPERTYbounds:
				propVal = null;
				if(window.mozInnerScreenX != undefined)
                {
					if(log) console.log("JSHtmlDocument::getBounds");
					var left = window.mozInnerScreenX,
                    top = window.mozInnerScreenY,
                    w = window.innerWidth,
                    h = window.innerHeight;
					var rect = {x:left, y:top, width:w, height:h};
					if(log) console.log("Returning rect as  "+ rect.toString());
					return [rect.x,rect.y,rect.width,rect.height];
                }
				else if ( isFTEdge )
                {
					if(log) console.log(" EDGE : JSHtmlDocument::getBounds");
					var mainWindow = window.top;
					var mainDocument = mainWindow.document;
					var left = mainWindow.screenLeft || mainWindow.screenX || 0;
                    var top = mainWindow.screenTop || mainWindow.screenY || 0;
                    var width =  mainDocument.defaultView.innerWidth || mainDocument.documentElement.clientWidth || mainDocument.body.clientWidth || 0;
                    var height = mainDocument.defaultView.innerHeight || mainDocument.documentElement.clientHeight || mainDocument.body.clientHeight || 0;
					if(log) console.log("EDGE Returning bounds - " + left + ", " + top + ", " + width + ", " + height);
					return [left, top, width, height];
                }
                break;
			case PropertySet.PROPERTYtitle:
				propVal = this.obj.title;
				break;
			case PropertySet.PROPERTYurl:
				propVal = this.obj.URL;
				break;
			case PropertySet.INTERNALPROPERTYbrowserWinHandle:
				var topWin = window.top;
				if(topWin) {
					propVal = new JSHtmlBrowser(topWin).getNativeHandle();
				}
				break;
			case PropertySet.PROPERTYtext:
				var docElem = this.obj.documentElement;
				if(docElem) {
					var docElement = JSHtmlDocument.prototype.create(docElem);
					retval = docElement.getProperty(propertyName);
				}
				break;
			case PropertySet.PROPERTYdomain:
				propVal = this.obj.domain;
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return null;
};

/*JSHtmlElement Proxy*/
var JSHtmlElement = function(obj) {
	JSHtmlObject.apply(this,obj);
	this.obj = obj;
};
JSHtmlElement.prototype = new JSHtmlObject();

JSHtmlElement.prototype.getDocument = function() {
	try {
		var docObj = this.obj.ownerDocument;
		if(docObj) {
			return new JSHtmlDocument(docObj);
		}
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.getDocumentHandle = function() {
	return this.getDocument();
};

JSHtmlElement.prototype.getTopHtmlParent = function() {
	try {
		var topWin = window.top;
		if(topWin) {
			return new JSHtmlBrowser(topWin).getNativeHandle();
		}
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.getParent = function() {
	try {
		var parentNode = this.obj.parentNode;
		if(parentNode && parentNode.tagName) {
			return JSHtmlDocument.prototype.create(parentNode);
		}
	} catch(err){}
	return this.getDocument();
};

JSHtmlElement.prototype.getChildAtPoint = function(x, y) {
	return null;
};

JSHtmlElement.prototype.getElementsByTag = function(tag) {
	try {
		var elems = this.obj.getElementsByTagName(tag);
		if(elems) {
			var elem, result = [];
			for(var i=0; i<elems.length; i++) {
				if(elem=elems[i]) {
					result.push(JSHtmlDocument.prototype.create(elem));
				}
			}
			return result;
		}
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.getElementWithFocus = function() {
	return null;
};

JSHtmlElement.prototype.getChildren = function() {
	try {
		var child, nodes = [];
		var childNodes = this.obj.childNodes;
		if(childNodes) {
			for(var i=0; i<childNodes.length; i++) {
				// nodeType 3 represents text node, which is the actual Text of an Element or Attribute
				if( (child=childNodes[i]) && (child.tagName || child.nodeType==3) ) {
					nodes.push(JSHtmlDocument.prototype.create(child));
				}
			}
			return nodes;
		}
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.scrollIntoView = function() {
	return this.scrollIntoViewWithDirection(false);
};

JSHtmlElement.prototype.scrollIntoViewWithDirection = function(direction) {
	try {
		this.obj.scrollIntoView(direction);
		return true;
	} catch(err){}
	return false;
};

JSHtmlElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		this.obj[propName] = value;
		return true;
	} catch(err){}
	return false;
};

JSHtmlElement.prototype.clickWithOutMouse = function() {
	return this.clickWithOutMouseAtPoint(0,0,0);
};

JSHtmlElement.prototype.clickWithOutMouseAtPoint = function(mousebutton, x, y) {
	try {
		if(mousebutton < 0 || mousebutton > 2) {
			return false;
		}
		var doc = this.getDocument().obj;
		var clickEvent = doc.createEvent("MouseEvents");
		clickEvent.initMouseEvent("click",true,true,doc.defaultView,1,x,y,0,0,false,false,false,false,mousebutton,null);
		this.obj.dispatchEvent(clickEvent);
		return true;
	} catch(err){}
	return false;
};

JSHtmlElement.prototype.getElementsByXpath = function(xpath) {
	return this.getAllElementsByXpath(xpath, this.obj, this.getDocument().obj);
};

JSHtmlElement.prototype.getAllElementsByXpath = function(xpath, contextObj, doc) {
	try {
		var resNode, child = [];
		var result = doc.evaluate(xpath, contextObj, null, XPathResult.ANY_TYPE, null);
		if(result) {
			while(resNode=result.iterateNext()){
				child.push(JSHtmlDocument.prototype.create(resNode));
			}
		}
		return child;
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.getElementsByJQuery = function(jquery) {
	return this.getAllElementsByJQuery(jquery, this.obj, this.getDocument().obj);
};

JSHtmlElement.prototype.getAllElementsByJQuery = function(jquery, contextObj, doc) {
	try {
		var resNode, child = [];
		var result = doc.defaultView.jQuery(jquery,contextObj);
		if(result) {
			for(var i=0;i<result.length;i++){
				if(resNode=result[i]) {
					child.push(JSHtmlDocument.prototype.create(resNode));
				}
			}
		}
		return child;
	} catch(err){}
	return null;
};

JSHtmlElement.prototype.getProperties = function() {
	var node, props = [];
	try {
		var attributes = this.obj.attributes;
		for(var i=0;i<attributes.length;i++) {
			if(node = attributes.item(i)) {
				props.push(node.nodeName);
			}
		}
	} catch(err){}
	return props;
};

JSHtmlElement.prototype.getStandardProperties = function() {
	return this.getStandardPropertynames();
};

JSHtmlElement.prototype.isPointInObject = function(x, y) {
	return this.isPointFoundInObject(x, y);
};

JSHtmlElement.prototype.getBounds = function() {
	var rect = {x:0, y:0, width:0, height:0};
	try {
		if(this.obj) {
			var bounds = this.obj.getBoundingClientRect();
			rect.x = bounds.left;
			rect.y = bounds.top;
			rect.width = bounds.right - rect.x;
			rect.height = bounds.bottom - rect.y;

			var zoomFactor = JSHtmlBrowser.prototype.getZoomingFactor(window.top);
			var offsetPt = JSHtmlDocument.prototype.convertFromOffsetToScreen(this.getDocument(), {x:rect.x, y:rect.y}, zoomFactor);
			rect.x = offsetPt.x;
			rect.y = offsetPt.y;
			rect.width = parseInt(rect.width * zoomFactor);
			rect.height = parseInt(rect.height * zoomFactor);
		}
	} catch(err) {}
	return rect;
};

JSHtmlElement.prototype.hasFocus = function() {
	try {
		var doc = this.obj.ownerDocument;
		if(doc) {
			var elem = doc.activeElement;
			try {
				if(elem) {
					return elem.isSameNode(this.obj);
				}
			} catch(err){}
		}
	} catch(err){}
	return false;
};

JSHtmlElement.prototype.isElementScrollable = function() {
	var isScrollable = false;
	try {
		if(this.isOverflowHiddenOrVisible(this.getProperty(".currentStyle.overflow"))) {
			if(this.isOverflowHiddenOrVisible(this.getProperty(".currentStyle.overflowX")) && this.isOverflowHiddenOrVisible(this.getProperty(".currentStyle.overflowY"))) {
				return false;
			}		
		}
		
		if( (((this.obj.clientHeight!=0) && (this.obj.scrollHeight > this.obj.offsetHeight))
				|| ((this.obj.clientWidth!=0) && (this.obj.scrollWidth > this.obj.offsetWidth)))
			&& ((this.obj.offsetHeight - this.obj.clientHeight - this.obj.clientTop > 5)
				|| (this.obj.offsetWidth - this.obj.clientWidth - this.obj.clientLeft > 5)) ) {
			isScrollable = true;
		}
	} catch(err){}
	return isScrollable;
};

JSHtmlElement.prototype.isOverflowHiddenOrVisible = function(propValue) {
	var result = false;
	if(propValue && (typeof propValue == "string")) {
		result = ((propValue=="hidden") || (propValue=="visible"));
	}
	return result;
};

JSHtmlElement.prototype.getScrollbarVWidth = function() {
	try {
		var width = 0;
		var offsetWidth = this.getIntVal("offsetWidth");
		var clientLeft = this.getIntVal("clientLeft");
		var clientWidth = this.getIntVal("clientWidth");

		if ((clientWidth > 0) && ((offsetWidth - clientLeft) > clientWidth)) {
			width = offsetWidth - clientWidth - clientLeft;
		}
		return width;
	} catch(err){}
	return 0;
};

JSHtmlElement.prototype.getScrollbarHHeight = function() {
	try {
		var height = 0;
		var offsetHeight = this.getIntVal("offsetHeight");
		var clientTop = this.getIntVal("clientTop");
		var clientHeight = this.getIntVal("clientHeight");

		if ((clientHeight > 0) && (offsetHeight - clientTop > clientHeight)) {
			height = offsetHeight - clientHeight - clientTop;
		}
		return height;
	} catch(err){}
	return 0;
};

JSHtmlElement.prototype.getXpathExpression = function() {
	var fullPath = [];	
	var el = this.obj;
	do {
		var index = 1;
		if (el.parentNode && !el.id) {
			var siblings = el.parentNode.childNodes;
			for (var i = 0; i < siblings.length; i++) {
				var sibling = siblings[i];
				if (sibling === el)
					break;
				if (sibling.nodeType === 1 && sibling.tagName === el.tagName)
					index++;
			}
		}
		fullPath.unshift(el.nodeName.toLowerCase() + (el.id ? '[@id=\'' + el.id + '\']' : '') + (index > 1 ? '[' + index + ']' : ''));
	} while (!el.id && ((el.nodeName.toLowerCase() != 'body') && el.parentNode && (el = el.parentNode)));
	return '//' + fullPath.join('/');
};

JSHtmlElement.prototype.getHierarchy = function() {
	var fullPath = [];	
	var el = this.obj;
	do {		
		fullPath.unshift(el.nodeName.toLowerCase());
	} while ((el.nodeName.toLowerCase() != 'body') && el.parentNode && (el = el.parentNode));
	return '//' + fullPath.join('/');
};

JSHtmlElement.prototype.getStyleProperty = function(propName) {
	var retValue = null;
	var styleSubProp = this.getStyleSubPropertyName(propName);
	if(styleSubProp) {
		try {
			var styleElem = this.obj.style;
			if(styleElem) {
				retValue = styleElem[styleSubProp];
			}
		} catch(err){}
	}
	return retValue;
}

JSHtmlElement.prototype.getStyleSubPropertyName = function(propName) {
	var styleProp = null;
	try {
		if((propName.indexOf("style")>=0) || (propName.indexOf("Style")>=0)) {
			var index = propName.lastIndexOf('.');
			if(index > 0) {
				styleProp = propName.substring(index+1, propName.length);
			}
		}
	} catch(err){}
	return styleProp;
}

JSHtmlElement.prototype.getNativeProperty = function(propName) {
	var retval = null;
	try {
		var attributes = this.obj.attributes;
		if(attributes) {
			retval = attributes.getNamedItem(propName);
			if(retval) {
				retval = retval.nodeValue;
				if(retval && !isNaN(retval)) {
					retval = parseInt(retval);
				}
			}
		}
	} catch(err){}	
	return retval;
}

JSHtmlElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName == "class") {
			propName = propName + "Name";
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		if(propName.startsWith("data-"))
		{
			var propNamesList = propName.substring(5, propName.length);
			propNamesList = propNamesList.split("-");
			var dataPropName = propNamesList[0];
			for (var i = 1; i < propNamesList.length; i++) 
			{
				dataPropName = dataPropName + propNamesList[i].charAt(0).toUpperCase() + propNamesList[i].slice(1)
			}
			propVal = this.obj.dataset[dataPropName];
			if(propVal != null && propVal != undefined)
			{
				return propVal;
			}
		}
		switch(propName) {
			case PropertySet.PROPERTYclass:
				propVal = "Html." + this.obj.tagName;
				break;
			case PropertySet.PROPERTYtag:
				propVal = this.obj.tagName;
				break;
			case PropertySet.PROPERTYid:
				propVal = this.obj.id;
				break;
			case PropertySet.PROPERTYname:
				propVal = this.obj.name;
				break;
			case PropertySet.PROPERTYtype:
				propVal = this.obj.type;
				break;
			case PropertySet.PROPERTYtitle:
				propVal = this.obj.title;
				break;
			case PropertySet.PROPERTYclassName:
				propVal = this.getStrVal(this.obj.className);
				break;
			case PropertySet.PROPERTYvalue:
				propVal = this.obj.value;
				break;
			case PropertySet.PROPERTYscreenLeft:
				var bounds = this.getBounds();
				propVal = bounds.x;
				break;
			case PropertySet.PROPERTYscreenTop:
				var bounds = this.getBounds();
				propVal = bounds.y;
				break;
			case PropertySet.PROPERTYoffsetWidth:
				propVal = this.getIntVal("offsetWidth");
				break;
			case PropertySet.PROPERTYoffsetHeight:
				propVal = this.getIntVal("offsetHeight");
				break;
			case PropertySet.PROPERTYoffsetLeft:
				propVal = this.getIntVal("offsetLeft");
				break;
			case PropertySet.PROPERTYoffsetTop:
				propVal = this.getIntVal("offsetTop");
				break;
			case PropertySet.INTERNALPROPERTYuniqueId:
				propVal = this.getNativeHandle().toString();
				break;
			case PropertySet.PROPERTYhasFocus:
				propVal = this.hasFocus();
				break;
			case PropertySet.INTERNALPROPERTYvScrollWidth:
				propVal = this.getScrollbarVWidth();
				break;
			case PropertySet.INTERNALPROPERTYhScrollHeight:
				propVal = this.getScrollbarHHeight();
				break;
			case PropertySet.PROPERTYxpath:
				propVal = this.getXpathExpression();
				break;
            case PropertySet.PROPERTYhierarchy:
				propVal = this.getHierarchy();
				break;
			case PropertySet.PROPERTYdisabled:
				propVal = this.getBoolVal("isDisabled");
				break;	
			case PropertySet.INTERNALPROPERTYbrowserWinHandle:
				var topWin = window.top;
				if(topWin) {
					propVal = new JSHtmlBrowser(topWin).getNativeHandle();
				}
				break;
			case PropertySet.INTERNALPROPERTYisScrollable:
				propVal = this.isElementScrollable();
				break;
			case PropertySet.PROPERTYcontentText:
				propVal = this.obj.textContent;
				break;
			case PropertySet.PROPERTYtext:
				propVal = this.obj.textContent.replace(/\s/g, ' ').replace(/ +/g, ' ').replace(/^ /g, ' ').replace(/ $/g, ' ');
				break;	
			case PropertySet.PROPERTYinnerText:
				propVal = this.obj.innerText.replace(/\s/g, ' ').replace(/ +/g, ' ').replace(/^ /g, ' ').replace(/ $/g, ' ');
				break;
			case PropertySet.PROPERTYbounds:
				var bounds = this.getBounds();
				propVal = [bounds.x, bounds.y, bounds.width, bounds.height];
				break;
			case PropertySet.PROPERTYmaxLength:
				propVal = this.getIntVal("maxLength");
				break;
			case PropertySet.PROPERTYmaxlength:
				propVal = this.getIntVal("maxLength");
				break;
			default:
				if(!this.isStandardProp(propName)) {
					propVal = this.getStyleProperty(propName);
					if(propVal != null) {
						propVal = this.getNativeProperty(propName);
						if(propVal != null) {
							propVal = this.obj[propName];
							if(propVal && !isNaN(propVal)) {
								propVal = parseInt(propVal);
							}
						}
					}
				}
			if(propVal == null)
			{
				propVal = this.getNativeProperty(propName);
			}
			if(propVal == null)
			{
				propVal = this.obj[propName];
			}
		}
		if(propVal == undefined || propVal instanceof window.Element) { // property shouldn't return an html element, returning null
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return null;
};

/*JSHtmlAnchorElement Proxy*/
var JSHtmlAnchorElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlAnchorElement.prototype = new JSHtmlElement();

JSHtmlAnchorElement.prototype._getProperty = JSHtmlAnchorElement.prototype.getProperty;
JSHtmlAnchorElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYname:
				propVal = this.obj.name;
				break;
			case PropertySet.PROPERTYhref:
				propVal = this.obj.href;
				break;
			case PropertySet.PROPERTYtext:
				propVal = this.obj.text;
				break;	
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

/*JSHtmlAreaElement Proxy*/
var JSHtmlAreaElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlAreaElement.prototype = new JSHtmlElement();

JSHtmlAreaElement.prototype._getProperty = JSHtmlAreaElement.prototype.getProperty;
JSHtmlAreaElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYalt:
				propVal = this.obj.alt;
				break;
			case PropertySet.PROPERTYtarget:
				propVal = this.obj.target;
				break;
			case PropertySet.PROPERTYnoHref:
				propVal = this.obj.noHref;
				break;
			case PropertySet.PROPERTYhref:
				propVal = this.obj.href;
				break;
			case PropertySet.PROPERTYcoords:
				propVal = this.obj.coords;
				break;
			case PropertySet.PROPERTYshape:
				propVal = this.obj.shape;
				break;		
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

/*JSHtmlImageElement Proxy*/
var JSHtmlImageElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlImageElement.prototype = new JSHtmlElement();

JSHtmlImageElement.prototype._getProperty = JSHtmlImageElement.prototype.getProperty;
JSHtmlImageElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYalign:
				propVal = this.obj.align;
				break;
			case PropertySet.PROPERTYborder:
				propVal = this.getIntVal("border");
				break;
			case PropertySet.PROPERTYheight:
				propVal = this.getIntVal("height");
				break;
			case PropertySet.PROPERTYwidth:
				propVal = this.getIntVal("width");
				break;
			case PropertySet.PROPERTYisMap:
				propVal = this.getBoolVal("isMap");
				break;
			case PropertySet.PROPERTYuseMap:
				propVal = this.obj.useMap;
				break;
			case PropertySet.PROPERTYhspace:
				propVal = this.getIntVal("hspace");
				break;
			case PropertySet.PROPERTYvspace:
				propVal = this.getIntVal("vspace");
				break;
			case PropertySet.PROPERTYsrc:
				propVal = this.obj.src;
				break;
			case PropertySet.PROPERTYalt:
				propVal = this.obj.alt;
				break;
			case PropertySet.PROPERTYname:
				propVal = this.obj.name;
				break;		
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlImageElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		if(propName == PropertySet.INTERNALPROPERTYscrollPosX) {
			this.obj[propName] = value;
			return true;
		}
	} catch(err){}
	return false;
};

/*JSHtmlInputElement Proxy*/
var JSHtmlInputElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlInputElement.prototype = new JSHtmlElement();

JSHtmlInputElement.prototype._getProperty = JSHtmlInputElement.prototype.getProperty;
JSHtmlInputElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName == "class") {
			propName = propName + "Name";
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYclass:
				propVal = "Html." + this.obj.tagName;
				var type = this.obj.type;
				if(type) {
					propVal = propVal + "." + type;
				}
				break;
			case PropertySet.PROPERTYtext:
				propVal = this.obj.textContent.replace(/\s/g, ' ').replace(/ +/g, ' ').replace(/^ /g, ' ').replace(/ $/g, ' ').trim();
				break;
			case PropertySet.PROPERTYvalue:
				propVal = this.obj.value;
				break;
			case PropertySet.PROPERTYtype:
				propVal = this.obj.type;
				break;
			case PropertySet.PROPERTYchecked:
				propVal = this.getBoolVal("checked");
				break;
			case PropertySet.PROPERTYdefaultValue:
				propVal = this.obj.defaultValue;
				break;
			case PropertySet.PROPERTYdefaultChecked:
				propVal = this.obj.defaultChecked;
				break;
			case PropertySet.PROPERTYalt:
				propVal = this.obj.alt;
				break;
			case PropertySet.PROPERTYsize:
				propVal = this.getIntVal("size");
				break;	
			case PropertySet.PROPERTYsrc:
				propVal = this.obj.src;
				break;
			case PropertySet.PROPERTYdisabled:
				propVal = this.obj.disabled;
				break;
			case PropertySet.PROPERTYname:
				propVal = this.obj.name;
				break;
			case PropertySet.PROPERTYindeterminate:
				propVal = this.obj.indeterminate;
				break;	
			case PropertySet.PROPERTYreadOnly:
				propVal = this.obj.readOnly;
				break;			
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined){
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlInputElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		if(propName==PropertySet.PROPERTYchecked || propName==PropertySet.PROPERTYdisabled || propName==PropertySet.PROPERTYvalue) {
			this.obj[propName] = value;
			return true;
		}
	} catch(err){}
	return false;
};

/*JSHtmlTextAreaElement Proxy*/
var JSHtmlTextAreaElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlTextAreaElement.prototype = new JSHtmlElement();

JSHtmlTextAreaElement.prototype._getProperty = JSHtmlTextAreaElement.prototype.getProperty;
JSHtmlTextAreaElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYtext:
				propVal = this.obj.textContent;
				break;
			case PropertySet.PROPERTYvalue:
				propVal = this.obj.value;
				break;
			case PropertySet.PROPERTYdefaultValue:
				propVal = this.obj.value;
				break;
			case PropertySet.PROPERTYrows:
				propVal = this.getIntVal("rows");
				break;
			case PropertySet.PROPERTYdisabled:
				propVal = this.getBoolVal("DISABLED");
				break;
			case PropertySet.PROPERTYreadOnly:
				propVal = this.getBoolVal("READONLY");
				break;
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined){
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlTextAreaElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		if(propName == PropertySet.PROPERTYvalue) {
			this.obj[propName] = value;
		}
		return true;
	} catch(err){}
	return false;
};

/*JSHtmlSelectElement Proxy*/
var JSHtmlSelectElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlSelectElement.prototype = new JSHtmlElement();

JSHtmlSelectElement.prototype._getProperty = JSHtmlSelectElement.prototype.getProperty;
JSHtmlSelectElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYsize:
				propVal = this.getIntVal("size");
				break;
			case PropertySet.PROPERTYtype:
				propVal = this.obj.type;
				break;
			case PropertySet.PROPERTYlength:
				propVal = this.getIntVal("length");
				break;
			case PropertySet.PROPERTYmultiple:
				propVal = this.obj.multiple;
				break;
			case PropertySet.PROPERTYdisabled:
				propVal = this.obj.disabled;
				break;
			case PropertySet.PROPERTYselectedIndex:
				propVal = this.getIntVal("selectedIndex");
				break;	
			case PropertySet.PROPERTYname:
				propVal = this.obj.name;
				break;	
			case PropertySet.PROPERTYtext:
				propVal = this.obj.textContent.replace(/\s/g, ' ').replace(/ +/g, ' ').replace(/^ /g, ' ').replace(/ $/g, ' ').trim();
				break;			
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlSelectElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		if(propName == PropertySet.PROPERTYselectedIndex) {
			return this.selectOption(value);
		} else if (propName == PropertySet.PROPERTYmultiselectOption) {
			var optionNodes = this.obj.options;
			var child = optionNodes[value];
			var selectedValue = child.selected;
			if(selectedValue) {
				child.selected = false;
			} else {
				child.selected = true;
			}
			return true;
		}
	} catch(err){}
	return false;
};

JSHtmlSelectElement.prototype.selectOption = function(index) {
	try {
		if(!isNaN(index)) {
			var currentSelectedIndex = null;
			var ret = this.obj.selectedIndex;
			if(!isNaN(ret)) {
				currentSelectedIndex = ret;
			}
			this.obj.selectedIndex = index;
			this.dispatchEvt("MouseEvents", "click");
			if((currentSelectedIndex != null) && (currentSelectedIndex != index)) {
				this.dispatchEvt("HTMLEvents", "change");
			}
		}
		return true;
	} catch(err){}
	return false;
};

JSHtmlSelectElement.prototype.dispatchEvt = function(eventClass, eventName) {
	try	{
		var doc = this.obj.ownerDocument;
		var theEvent = doc.createEvent(eventClass);
		theEvent.synthetic = true;
		theEvent.initEvent(eventName, true, true);		
		this.obj.dispatchEvent(theEvent);
	} catch(err){}
};

/*JSHtmlOptionElement Proxy*/
var JSHtmlOptionElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlOptionElement.prototype = new JSHtmlElement();

JSHtmlOptionElement.prototype._getProperty = JSHtmlOptionElement.prototype.getProperty;
JSHtmlOptionElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYselected:
				propVal = this.getBoolVal("selected");
				break;
			case PropertySet.PROPERTYindex:
				propVal = this.getIntVal("index");
				break;
			case PropertySet.PROPERTYtext:
				propVal = this.obj.textContent;
				break;
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlOptionElement.prototype.getBounds = function() {
	try {
		var rect = {};
		var bounds = this.obj.getBoundingClientRect();
		rect.x = bounds.left;
		rect.y = bounds.top;
		rect.height = bounds.bottom - bounds.top;
		rect.width = bounds.right - bounds.left;
		return rect;
	} catch(err){}
	return {x:0,y:0,width:0,height:0};
};

/*JSHtmlTableElement Proxy*/
var JSHtmlTableElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlTableElement.prototype = new JSHtmlElement();

JSHtmlTableElement.prototype._getProperty = JSHtmlTableElement.prototype.getProperty;
JSHtmlTableElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName == "class") {
			propName = propName + "Name";
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYborder:
				propVal = this.obj.border;
				break;
			case PropertySet.PROPERTYcaption:
				propVal = this.obj.caption;
				if(!propVal || propVal instanceof window.Element) { // property shouldn't return an html element, returning null
					propVal = "";
				}
				break;
			case PropertySet.PROPERTYwidth:
				propVal = this.obj.width;
				break;		
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined){
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

/*JSHtmlTableCellElement Proxy*/
var JSHtmlTableCellElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlTableCellElement.prototype = new JSHtmlElement();

JSHtmlTableCellElement.prototype.getTableCellData = function(node, text) {
	try {
		var DOMElementNode = 1 ;
		var DOMTextNode = 3 ;
		var domNodeType = node.nodeType;
		switch(domNodeType)
		{
			case DOMTextNode:
				var objText = node.nodeValue;
				text += objText;
				break;
			case DOMElementNode:
				var numChildren = 0 ;
				var childNode = node.firstChild;
				while(childNode) {
					numChildren++;
					text = this.getTableCellData(childNode, text);				
					childNode = childNode.nextSibling;
				}
				if(numChildren == 0) {
					var tag = node.tagName;								
					if(tag) {
						switch(tag) {
							case "BR":
								text += " ";
								break;
							case "TITLE":
								text += node.text;
								text += " ";
								break;
							case "INPUT":
								var tagType = node.type;
								if((tagtype != "checkbox") && (tagtype != "radio")) {
									text += node.value;
									text += " ";
								}
								break;
						}
					}
				}
				break;
		}
		return text;
	} catch(err){}
	return text;
};

JSHtmlTableCellElement.prototype._getProperty = JSHtmlTableCellElement.prototype.getProperty;
JSHtmlTableCellElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYcellIndex:
				propVal = this.getIntVal("cellIndex");
				break;
			case PropertySet.PROPERTYwidth:
				propVal = this.getIntVal("width");
				break;
			case PropertySet.PROPERTYrowSpan:
				propVal = this.getIntVal("rowSpan");
				break;
			case PropertySet.PROPERTYtableCellData:
				var text = "";
				propVal = this.getTableCellData(this.obj, text);
				break;			
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

/*JSHtmlTableRowElement Proxy*/
var JSHtmlTableRowElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlTableRowElement.prototype = new JSHtmlElement();

JSHtmlTableRowElement.prototype._getProperty = JSHtmlTableRowElement.prototype.getProperty;
JSHtmlTableRowElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYrowIndex:
				propVal = this.getIntVal("rowIndex");
				break;
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

/*JSHtmlFrameElement Proxy*/
var JSHtmlFrameElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlFrameElement.prototype = new JSHtmlElement();

JSHtmlFrameElement.prototype._getProperty = JSHtmlFrameElement.prototype.getProperty;

JSHtmlFrameElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		propVal = this._getProperty(propertyName);
		if(propVal==undefined) {
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return this._getProperty(propertyName);
};

JSHtmlFrameElement.prototype.getChildren = function() {
	try {
		var frameDoc = this.getFrameDocument();
		if(frameDoc) {
			return [frameDoc];
		}
	} catch(err){}
	return null;
};

JSHtmlFrameElement.prototype.getElementWithFocus = function() {
	try {
		var frameDoc = this.getFrameDocument();
		if(frameDoc) {
			return frameDoc.getElementWithFocus();
		}
	} catch(err){}
	return null;
};

JSHtmlFrameElement.prototype.getFrameDocument = function() {
	try {
		var frameWin = this.obj.contentWindow;
		var frameDoc = this.obj.contentDocument;
		if(!frameDoc && frameWin) {
			frameDoc = frameWin.document;
		}
		if(frameDoc) {
			return new JSHtmlDocument(frameDoc);
		}
	} catch(err){}
	return null;
};

JSHtmlFrameElement.prototype.getFramesOffsetPoint = function() {
	var offstpt = null;
	try {
		var rect = this.obj.getBoundingClientRect();
		offstpt = {x:rect.left, y:rect.top};
		if(offstpt.y == 0) {
			var parent = this.getParent();
			var grparent = parent.getParent();
			if(grparent && (grparent instanceof JSHtmlFrameSetElement)) {
				var parsib = parent.previousElementSibling;
				if(parsib) {
					var parsibelem = JSHtmlDocument.prototype.create(parsib);
					if(parsibelem instanceof JSHtmlFrameElement) {
						offstpt.y = parsib.height;
					}
				}
			}
		}
	} catch(err){}
	return offstpt;
};

/*JSHtmlFrameSetElement Proxy*/
var JSHtmlFrameSetElement = function(obj) {
	JSHtmlElement.apply(this,obj);
	this.obj = obj;
};
JSHtmlFrameSetElement.prototype = new JSHtmlElement();

/*JSHtmlTextNodeElement Proxy*/
var JSHtmlTextNodeElement = function(obj) {
	JSHtmlObject.apply(this,obj);
	this.obj = obj;
};
JSHtmlTextNodeElement.prototype = new JSHtmlObject();

JSHtmlTextNodeElement.prototype.getNativeHandle = function() {
	return this.nativeHandle;
};

JSHtmlTextNodeElement.prototype.setNativeHandle = function(handle) {
	this.nativeHandle = handle;
};

JSHtmlTextNodeElement.prototype.getElementsByTag = function(name) {
	return null;
};

JSHtmlTextNodeElement.prototype.getDocument = function() {
	try {
		var docObj = this.obj.ownerDocument;
		if(docObj) {
			return new JSHtmlDocument(docObj);
		}
	} catch(err){}
	return null;
};

JSHtmlTextNodeElement.prototype.getParent = function() {
	try {
		var parentNode = this.obj.parentNode;
		if(parentNode && parentNode.tagName) {
			return JSHtmlDocument.prototype.create(parentNode);
		}
	} catch(err){}
	return this.getDocument();
};

JSHtmlTextNodeElement.prototype.getTopHtmlParent = function() {
	try {
		var topWin = window.top;
		if(topWin) {
			return new JSHtmlBrowser(topWin).getNativeHandle();
		}
	} catch(err){}
	return 0;
};

JSHtmlTextNodeElement.prototype.getChildren = function(handle) { 
	return null;
};

JSHtmlTextNodeElement.prototype.getNativeProperty = function(handle) { 
	var retval = null;
	try {
		var attributes = this.obj.attributes;
		if(attributes) {
			retval = attributes.getNamedItem(propName);
			if(retval) {
				retval = retval.nodeValue;
				if(retval && !isNaN(retval)) {
					retval = parseInt(retval);
				}
			}
		}
	} catch(err){}	
	return retval;
};

JSHtmlTextNodeElement.prototype.getProperty = function(propertyName) {
	var propName = propertyName, propVal = null;
	if(!propName) {
		return null;
	}
	try {
		if(propName == "class") {
			propName = propName + "Name";
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		switch(propName) {
			case PropertySet.PROPERTYtext:
				propVal = this.obj.nodeValue;
				break;
			case PropertySet.PROPERTYclass:
				propVal = "Html.TextNode";
				break;
			default:
				propVal = this._getProperty(propertyName);
				break;
		}
		if(propVal==undefined){
			propVal = null;
		}
		return propVal;
	} catch(err){}
	return null;
};

JSHtmlTextNodeElement.prototype.getProperties = function() {
	return null;
};

JSHtmlTextNodeElement.prototype.getStandardProperties = function() {
	return null;
};

JSHtmlTextNodeElement.prototype.setProperty = function(propName, value) {
	try {
		if(!propName) {
			return false;
		}
		if(propName.startsWith('.')) {
			propName = propName.substring(1, propName.length);
		}
		this.obj[propName] = value;
		return true;
	} catch(err){}
	return false;
};

JSHtmlTextNodeElement.prototype.scrollIntoView = function() {
	return this.scrollIntoViewWithDirection(false);
};

JSHtmlTextNodeElement.prototype.scrollIntoViewWithDirection = function(direction) {
	try {
		this.obj.scrollIntoView(direction);
		return true;
	} catch(err){}
	return false;
};

JSHtmlTableElement.prototype.getRowAndColIndexes = function(x , y)
{
try
    {
        var doc = this.getDocument();
        var cell;
        var row;
        if(doc)
        {
            var elemAtPoint = doc.getChildAtPoint(x, y);
			while(elemAtPoint)
            {
                try
                {
					if(doc.obj.isEqualNode(elemAtPoint.obj))
                    {
                        break;
                    }
                    if(elemAtPoint.obj.tagName == "TD")
					{
						cell = elemAtPoint.obj;
                    } 
					else if(elemAtPoint.obj.tagName == "TR") 
					{
					    row = elemAtPoint.obj;
                    } 
					else if (elemAtPoint.obj.isEqualNode(this.obj)) 
					{
						return row.rowIndex+":"+cell.cellIndex;
                    } 
					else if (elemAtPoint.obj.tagName == "TABLE") 
					{
                        cell = null;
                        row = null;
                    }
                }
                catch(err)
                {
					
                }
                elemAtPoint = elemAtPoint.getParent();
            }
        }
    }
    catch(err)
    {
      
    }
	return "";
};

JSHtmlTextNodeElement.prototype.getDocumentHandle = function() {
	return null;
};

JSHtmlTextNodeElement.prototype.getChildAtPoint = function(x, y) {
	return null;
};

JSHtmlTextNodeElement.prototype.getElementWithFocus = function() {
	return null;
};

JSHtmlTextNodeElement.prototype.isPointInObject = function(x, y) {
	return false;
};

JSHtmlTextNodeElement.prototype.getElementsByXpath = function(xpath) {
	return null;
};

JSHtmlTextNodeElement.prototype.getElementsByJQuery = function(jquery) {
	return null;
};

/*Property Set*/
var PropertySet = {
	PROPERTYtag:"tag",
	PROPERTYid:"id",
	PROPERTYname:"name",
	PROPERTYtitle:"title",
	PROPERTYclass:"class",
	PROPERTYoffsetLeft:"offsetLeft",
	PROPERTYoffsetTop:"offsetTop",
	PROPERTYoffsetWidth:"offsetWidth",
	PROPERTYoffsetHeight:"offsetHeight",
	PROPERTYtext:"text",
	PROPERTYscreenLeft:"screenLeft",
	PROPERTYscreenTop:"screenTop",
	PROPERTYclassName:"className",
	PROPERTYbounds:"bounds",
	INTERNALPROPERTYuniqueId:"uniqueId",
	PROPERTYurl:"url",
	PROPERTYvalue:"value",
	PROPERTYtype:"type",
	PROPERTYchecked:"checked",
	PROPERTYdefaultValue:"defaultValue",
	PROPERTYdefaultChecked:"defaultChecked",
	PROPERTYalt:"alt",
	PROPERTYsize:"size",
	PROPERTYsrc:"src",
	PROPERTYdisabled:"disabled",
	PROPERTYmaxLength:"maxLength",
	PROPERTYmaxlength:"maxlength",
	PROPERTYlength:"length",
	PROPERTYmultiple:"multiple",
	PROPERTYselectedIndex:"selectedIndex",
	PROPERTYmultiselectOption:"multiselectOption",
	PROPERTYindeterminate:"indeterminate",
	PROPERTYselected:"selected",
	PROPERTYindex:"index",
	PROPERTYreadOnly:"readOnly",
	PROPERTYalign:"align",
	PROPERTYborder:"border",
	PROPERTYheight:"height",
	PROPERTYwidth:"width",
	PROPERTYhspace:"hspace",
	PROPERTYvspace:"vspace",
	PROPERTYisMap:"isMap",
	PROPERTYuseMap:"useMap",
	PROPERTYwindow:"window",
	PROPERTYshape:"shape",
	PROPERTYcoords:"coords",
	PROPERTYhref:"href",
	PROPERTYcaption:"caption",
	PROPERTYrowIndex:"rowIndex",
	PROPERTYcellIndex:"cellIndex",
	PROPERTYrowSpan:"rowSpan",
	PROPERTYtableCellData:"tableCellData",
	PROPERTYtarget:"target",
	PROPERTYnoHref:"noHref",
	PROPERTYreadyState:"readyState",
	PROPERTYdomain:"domain",
	PROPERTYrows:"rows",
	PROPERTYclientRect:"clientRect",
	INTERNALPROPERTYscrollPosX:"scrollPosX",
	PROPERTYdocumentName:"documentName",
	PROPERTYstatusText:"statusText",
	INTERNALPROPERTYvScrollWidth:"vScrollWidth",
	INTERNALPROPERTYhScrollHeight:"hScrollHeight",
	PROPERTYhasFocus:"hasFocus",
	INTERNALPROPERTYisScrollable:"isScrollable",
	PROPERTYcontentText:"contentText",
	PROPERTYbrowserName:"browserName",
	INTERNALPROPERTYbrowserWinHandle:"browserWinHandle",
	PROPERTYinnerText:"innerText",
	PROPERTYxpath:"xpath",
    PROPERTYhierarchy:"hierarchy",
	PROPERTYbrowserWinRect:"browserWinRect"
};