/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2016. All Rights Reserved.
 * (c) Copyright HCL Technologies Ltd. 2017, 2019. All Rights Reserved.
 * 
 * U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
var log = false;
var hashMap = {};
var isNew = false;
var isFirefox = false;
var randMax = Math.pow(10,10);

loginfo("rftmain.js - document url -> " + window.top.document.URL);

if(navigator.userAgent.toLowerCase().indexOf("firefox")!=-1) {
	isFirefox = true;
}

document.addEventListener('rft_request', function(event) {
	if(log) loginfo("rftmain rft_request : " + JSON.stringify(event.detail));
	var resp = invokeMethod(event.detail.methodName, event.detail.methodArgs);
	if(log) loginfo("rftmain rft_response : " + JSON.stringify(resp));
	document.dispatchEvent(new CustomEvent('rft_response', {detail: resp}));
});

function executeScript(json) {
	try {
		if(log) loginfo("executeScript json : " + json);
		json = JSON.parse(json);
		var resp = invokeMethod(json.methodName, json.methodArgs);
		resp = JSON.stringify(resp);
		return resp;
	} catch(err){
		console.error("Error in executeScript - " + err.message);
	}
	return JSON.stringify({retVal:null});
}

function getRowAndColIndexes(params)
{
	var obj, handle=params[0], x=params[1], y=params[2], json={};
	if(log) loginfo("getRowAndColIndexes called with param - " + handle + " , " + x + " , " + y);
	try {
		if(handle && (obj=hashMap[handle])) {
			json.retVal = obj.getRowAndColIndexes(x, y);
			return json;
		}
	} catch(err){}
	return {retVal:""};
}

document.addEventListener('rft_ff_request',function(evnt){
    var resp = "";
    var data = evnt.detail;
    var _json = JSON.parse(data);
	if(log)loginfo("document::rftmain:rft_ff_request: "+ _json.methodName+ ":"+ _json.methodArgs);
	resp = invokeMethod(_json.methodName, _json.methodArgs);    
    resp.isNew = isNew;
	isNew = false;
	if(log) loginfo("rftmain rft_response : " + JSON.stringify(resp));
	document.dispatchEvent(new CustomEvent('rft_response', {detail: resp}));  
});

function loginfo(message) {
	       console.log(message);
};

function invokeMethod(methodName,methodArgs) {
	try {
		var method = new Function("methodArgs", "return " + methodName + "(methodArgs)");
		return method(methodArgs);
	} catch(err){
		console.log(err);		
	}
	return {retVal:null};
};

function getRandomHandle() {
	var count = 0;
	do {
		randomHandle = Math.floor(Math.random()*randMax)+1;
	} while((hashMap[randomHandle]!=undefined) || (count++>100));
	if(log) loginfo("randomHandle is " + randomHandle);
	return randomHandle;
};

function addToHashMap(obj) {
	try {
		if(obj instanceof JSHtmlObject){
			var handle = obj.getNativeHandle();
			if(handle == 0)
			{
				handle = getRandomHandle();
				obj.setNativeHandle(handle);
				isNew = true;
			}
			if(hashMap[handle] == undefined) {
				hashMap[handle] = obj;
			}
			return handle;
		}
	} catch(err){}
	return 0;
};


function isAppActive() {
	if(log) loginfo("isAppActive called");
	var json = {};
	json.retVal = "true";
	return json;
};

function isValidJSWrapper() {
	if(log) loginfo("isValidJSWrapper called");
	var json = {};
	try {
		if(window.top.document) {
			json.retVal = true;
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function getBrowserHandle(params) {
	var handle=params[0], json={};
	if(log) loginfo("getBrowserHandle called with param - " + handle);
	try {
		if(handle) {
			if(hashMap[handle]) {
				json.retVal = handle;
			} else {
				var browser = new JSHtmlBrowser(window.top);
				browser.setNativeHandle(handle);
				json.retVal = addToHashMap(browser);
			}
		}
		return json;
	} catch(err){}
	return {retVal:0};
};

function getBrowserDocument(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getBrowserDocument called with param - " + handle);
	try {
		if(handle) {
			if(!(obj=hashMap[handle])) {
				obj = new JSHtmlBrowser(window.top);
				obj.setNativeHandle(handle);
			}
			if(obj instanceof JSHtmlBrowser) {
				json.retVal = addToHashMap(obj.getDocument());
				return json;
			}
		}
	} catch(err){}
	return {retVal:0};
};

function getDocumentHandle(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getDocumentHandle called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = addToHashMap(obj.getDocumentHandle());
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

function getTopHtmlParent(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getTopHtmlParent called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = obj.getTopHtmlParent();
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

function getParent(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getParent called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = addToHashMap(obj.getParent());
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

var cachedFtDoc = {};
function getChildAtPoint(params) {
	var obj, handle=params[0], x=params[1], y=params[2], json={};
	if(log) loginfo("getChildAtPoint called with param - " + handle + " , " + x + " , " + y);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = addToHashMap(obj.getChildAtPoint(x,y));
			cachedFtDoc.doc = {};
			cachedFtDoc.browser = {};
			var elem = hashMap[json.retVal];
			var elemDoc = elem.getDocument();
			var docBrowser = elemDoc.getParent();
			cachedFtDoc.doc.handle = addToHashMap(elemDoc);
			cachedFtDoc.doc.title = elemDoc.getProperty(".title");
			cachedFtDoc.doc.url = elemDoc.getProperty(".url");
			cachedFtDoc.doc.xpath = elem.getXpathExpression();
			cachedFtDoc.browser.handle = docBrowser.obj.FtNativeHandle;
			if(isFirefox) {
				window.localStorage["cachedFtDoc"] = JSON.stringify(cachedFtDoc);
			}
			if(log) loginfo("getChildAtPoint cachedFtDoc: " + JSON.stringify(cachedFtDoc));
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

function getProperty(params) {
	var obj, handle=params[0], propName=params[1], json={};
	if(log) loginfo("getProperty called with param - " + handle + " , " + propName);
	try {
		if(handle && propName && (obj=hashMap[handle])) {
			json.retVal = obj.getProperty(propName);
			return json;
		}
		else if(propName==".readyState" || propName=="readyState") {
			var newdDoc = new JSHtmlDocument(window.top.document);			
			json.retVal = newdDoc.getReadyState();
			return json;
		}
	} catch(err){}
	return {retVal:null};
};
/**
JSON {"properties" : [  {"propertyName": "id"},{ "proprtyName":"name"} ] }
**/
function fetchProperties(params){  
    var obj, handle = params[0];
    var count = params.length;
    var propValues = [];
    if(handle && (obj=hashMap[handle]))
    {
    	var ret;
        for(var i = 1 ; i <count ;  i++)
        {
         	ret = obj.getProperty(params[i]);        
          	propValues.push(ret);
        }
    }     
    var json = {};
    json.retVal = propValues;
    return json;    
};

function getElementById(params) {
	var obj, handle=params[0], id=params[1], json={};
	if(log) loginfo("getElementById called with param - " + handle + " , " + id);
	try {
		if(handle && id && (obj=hashMap[handle]) && (obj instanceof JSHtmlDocument)) {
			json.retVal = addToHashMap(obj.getElementById(id));
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

function getElementWithFocus(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getElementWithFocus called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = addToHashMap(obj.getElementWithFocus());
			return json;
		}
	} catch(err){}
	return {retVal:0};
};

function getElementsByName(params) {
	var obj, handle=params[0], name=params[1], json={};
	if(log) loginfo("getElementsByName called with param - " + handle + " , " + name);
	try {
		if(handle && name && (obj=hashMap[handle]) && (obj instanceof JSHtmlDocument)) {
			var node, child = [];
			var nodes = obj.getElementsByName(name);
			if(nodes) {
				for(var i=0; i<nodes.length; i++) {
					if(node=nodes[i]) {
						child.push(addToHashMap(node));
					}
				}
			}
			json.retVal = child;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getElementsByTag(params){
	var obj, handle=params[0], tag=params[1], json={};
	if(log) loginfo("getElementsByTag called with param - " + handle + " , " + tag);
	try {
		if(handle && tag && (obj=hashMap[handle])){
			var node, child = [];
			var nodes = obj.getElementsByTag(tag);
			if(nodes) {
				for(var i=0; i<nodes.length; i++) {
					if(node=nodes[i]) {
						child.push(addToHashMap(node));
					}
				}
			}
			json.retVal = child;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getElementsByXpath(params) {
	var obj, handle=params[0], xpath=params[1], json={};
	if(log) loginfo("getElementsByXpath called with param - " + handle + " , " + xpath);
	try {
		if(handle && xpath && (obj=hashMap[handle])){
			var node, child = [];
			var nodes = obj.getElementsByXpath(xpath);
			if(nodes) {
				for(var i=0; i<nodes.length; i++) {
					if(node=nodes[i]) {
						child.push(addToHashMap(node));
					}
				}
			}
			json.retVal = child;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getElementsByJQuery(params) {
	var obj, handle=params[0], jquery=params[1], json={};
	if(log) loginfo("getElementsByJQuery called with param - " + handle + " , " + jquery);
	try {
		if(handle && jquery && (obj=hashMap[handle])) {
			var node, child = [];
			var nodes = obj.getElementsByJQuery(xpath);
			if(nodes) {
				for(var i=0; i<nodes.length; i++) {
					if(node=nodes[i]) {
						child.push(addToHashMap(node));
					}
				}
			}
			json.retVal = child;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getProperties(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getProperties called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])) {
			json.retVal = obj.getProperties();
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getStandardProperties(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getStandardProperties called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])) {
			json.retVal = obj.getStandardProperties();
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getChildren(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getChildren called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])) {
			var child = [];
			var nodes = obj.getChildren();
			if(nodes) {
				for(var i=0; i<nodes.length; i++) {
					if(nodes[i]) {
						child.push(addToHashMap(nodes[i]));
					}
				}
			}
			json.retVal = child;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getAllDocumentsInsideRecursively(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getAllDocumentsInsideRecursively called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle]) && (obj instanceof JSHtmlDocument)) {
			var docs = [];
			var allDocs = obj.getAllDocumentsInsideRecursively();
			for(var i=0; i<allDocs.length; i++) {
				docs.push(addToHashMap(allDocs[i]));
			}
			json.retVal = docs;
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function getAllDocumentsInsideFrame(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("getAllDocumentsInsideFrame called with param - " + handle);
	try {
		if(handle) {
			if(!(obj=hashMap[handle])) {
				obj = new JSHtmlBrowser(window.top);
				obj.setNativeHandle(handle);
				addToHashMap(obj);
			}
			var frameDoc = null;
			if(obj instanceof JSHtmlFrameElement) {
				frameDoc = obj.getFrameDocument();
			} else if(obj instanceof JSHtmlBrowser) {
				frameDoc = obj.getDocument();
			} else if(obj instanceof JSHtmlElement) {
				frameDoc = obj.getDocument();
			}
			if(frameDoc) {
				var docs = [];
				var allDocs = frameDoc.getAllDocumentsInsideRecursively();
				for(var i=0; i<allDocs.length; i++) {
					docs.push(addToHashMap(allDocs[i]));
				}
				if(obj instanceof JSHtmlFrameElement) {
					docs.push(addToHashMap(frameDoc));
				}
				json.retVal = docs;
				return json;
			}
		}
	} catch(err){}
	return {retVal:null};
};

function go(params) {
    setTimeout(function(){
        var obj, handle=params[0], direction=params[1], json={};
	if(log) loginfo("go called with param - " + handle + " , " + direction);
	try {
		if(handle && (obj=hashMap[handle]) && (obj instanceof JSHtmlBrowser)){
			json.retVal = obj.go(direction);
			return json;
		}
	} catch(err){}
    },10);
	return {retVal:false};
};

function loadUrl(params) {
	var obj, handle=params[0], url=params[1], target=params[2], json={};
	if(log) loginfo("loadUrl called with param - " + handle + " , " + url + " , " + target);
	try {
		if(handle && url && (obj=hashMap[handle]) && (obj instanceof JSHtmlBrowser)){
			json.retVal = obj.loadUrl(url, target);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function isPointInObject(params) {
	var obj, handle=params[0], x=params[1], y=params[2], json={};
	if(log) loginfo("isPointInObject called with param - " + handle + " , " + x + " , " + y);
	try {
		if(handle && (obj=hashMap[handle])) {
			json.retVal = obj.isPointInObject(x, y);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function scrollIntoView(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("scrollIntoView called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = obj.scrollIntoView();
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function scrollIntoViewWithDirection(params) {
	var obj, handle=params[0], direction=params[1], json={};
	if(log) loginfo("scrollIntoViewWithDirection called with param - " + handle + " , " + direction);
	try {
		if(handle && (obj=hashMap[handle])){
			json.retVal = obj.scrollIntoViewWithDirection(direction);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function invokeScript(params) {
	var obj, handle=params[0], method=params[1], scriptLang=params[2], json={};
	if(log) loginfo("invokeScript called with param - " + handle + " , " + method + " , " + scriptLang);
	try {
		if(handle && (obj=hashMap[handle]) && (obj instanceof JSHtmlDocument)) {
			json.retVal = obj.invokeScript(method, scriptLang);
			return json;
		}
	} catch(err){}
	return {retVal:null};
};

function setProperty(params) {
	var obj, handle=params[0], propName=params[1], propVal=params[2], json={};
	if(log) loginfo("setProperty called with param - " + handle + " , " + propName + " , " + propVal);
	try {
		if(handle && propName && (obj=hashMap[handle])) {
			json.retVal = obj.setProperty(propName, propVal);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function clickWithOutMouse(params) {
	var obj, handle=params[0], json={};
	if(log) loginfo("clickWithOutMouse called with param - " + handle);
	try {
		if(handle && (obj=hashMap[handle]) && (obj instanceof JSHtmlElement)) {
			json.retVal = obj.clickWithOutMouse();
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function clickWithOutMouseAtPoint(params) {
	var obj, handle=params[0], mouseButton=params[1], x=params[2], y=params[3], json={};
	if(log) loginfo("clickWithOutMouseAtPoint called with param - " + handle + " , " + mouseButton + " , " + x + " , " + y);
	try {
		if(handle && (obj=hashMap[handle]) && (obj instanceof JSHtmlElement)) {
			json.retVal = obj.clickWithOutMouseAtPoint(mouseButton, x, y);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};

function isEqual(params){
	var obj, source, target, sourceHandle=params[0], targetHandle=params[1], json={};
	if(log) loginfo("isEqual called with param - " + sourceHandle + " , " + targetHandle);
	try {
		if(sourceHandle && targetHandle && (source=hashMap[sourceHandle]) && (target=hashMap[targetHandle])) {
			json.retVal = source.obj.isSameNode(target.obj);
			return json;
		}
	} catch(err){}
	return {retVal:false};
};
function getFrameXpath(params){
	var fullPath = [];	
	var el = params;
	if(el == null)
		{
		 return "";
		}
	do {
		var index = 1;
		if (el.parentNode && !el.id) {
			var siblings = el.parentNode.childNodes;
			for (var i = 0; i < siblings.length; i++) {
				var sibling = siblings[i];
				if (sibling === el)
					break;
				if (sibling.nodeType === 1 && sibling.tagName === el.tagName)
					index++;
			}
		}
		if(el.nodeName.toLowerCase() != '#document')
			{
			fullPath.unshift(el.nodeName.toLowerCase() + (el.id ? '[@id=\'' + el.id + '\']' : '') + (index > 1 ? '[' + index + ']' : ''));
			}		
	} while (!el.id && ((el.nodeName.toLowerCase() != 'body') && el.parentNode && (el = el.parentNode)));
	return '//' + fullPath.join('/');
};
// this method is used only during rtww playback
// to set the found element in document for rtww to invoke action
function setElementInDocument(params) {
	var element, handle=params[0];
	var listOfObjects = [];
	var i = 0;
	if(log) loginfo("setElementInDocument called with param - " + handle);
	try {
		if( handle && (element=hashMap[handle]) ) {
			var mainDoc = window.top.document;
			mainDoc.rmotElement = element.obj;
			var ratio = window.top.devicePixelRatio ? window.top.devicePixelRatio : 1;
			var elemBounds = element.getBounds();
			mainDoc.rmotElementBounds = parseInt(elemBounds.x / ratio) + "," + parseInt(elemBounds.y / ratio);
			mainDoc.rmotElementXpath = element.getXpathExpression();
			mainDoc.rmotElementframe = element.obj.ownerDocument.defaultView.window.frameElement;
			while(mainDoc.rmotElementframe!=null)
				{
				listOfObjects[i] = getFrameXpath(mainDoc.rmotElementframe);
				mainDoc.rmotElementframe = mainDoc.rmotElementframe.ownerDocument.defaultView.window.frameElement;
				i=i+1;
				}
			mainDoc.listOfObjects = listOfObjects;
			mainDoc.stepId = handle;
			return {retVal:true};
		}
	} catch(err){}
	return {retVal:false};
};